PROC CopyFld()
;###############################################################################
;# CopyFld 1.3 (C) 1990, 1991, 1992 Data Works of Houston. All rights reserved #
;#   Written by: Cary Jensen of Data Works of Houston                          #
;#               (713) 359-3311                                                #
;#   Data Works of Houston specializes in Paradox and PAL consulting,          #
;#   Paradox and PAL training, and PAL applications.                           #
;#                                                                             #
;#   This program will permit you to copy and paste fields within and between  #
;#   Paradox Reports.                                                          #
;#   Here are two ways to play CopyFld:                                        #
;#      1.  Press [ALT][F10] to display the PAL menu.  Select {Play} from the  #
;#          the PAL menu and then type                                         #
;#          READLIB "C:\\PDOX35\\CopyTool" CopyFld CopyFld()                   #
;#          If you use this method, these steps must be repeated each time you #
;#          want to invoke CopyFld.  The following way is much easier.         #
;#      2.  Use the SETKEY command to allow you to play CopyFld by pressing a  #
;#          key sequence.  For example, to play CopyFld simply by pressing     #
;#          [CTRL][F3], you need to use the following steps only once during   #
;#          your Paradox session.                                              #
;#          Press [ALT][F10] to display the PAL menu.  Select MiniScript and   #
;#          enter the following command:                                       #
;#          SETKEY "F23" READLIB "C:\\PDOX35\\CopyTool" CopyFld CopyFld()      #
;#          Note: in the above command, dual backslashes (\\) are necessary.   #
;#          If you use an init script, add this command to it and CopyFld will #
;#          automatically be available every time you start Paradox.           #
;#   Both of these examples assume that you have placed CopyTool.lib onto the  #
;#   path C:\PDOX35.  If you store CopyTool in any other directory, supply     #
;#   that directory name instead of C:\\PDOX35 when using either of the        #
;#   above methods to play CopyFld.                                            #
;#                                                                             #
;# These following variables are not released.  They are used by the system    #
;# to remember what the copied field's contents are.                           #
;#     Decimals       = # of decimal places                                    #
;#     WrappedTimes   = # of lines wrapped onto                                #
;#     Justify        = type of justification                                  #
;#     Grouped        = is a summary field grouped                             #
;#     PlaceType      = type of field                                          #
;#     SummaryType    = is a summary field regular or calculated               #
;#     NegParens      = are negative values enclosed in parenthese             #
;#     StatType       = type of summary or report field                        #
;#     Commas         = are commas used                                        #
;#     NameOfField    = name of the field, calculation                         #
;#     Currency       = is the field a currency field                          #
;#     FieldFormat    = field string as it appears on the report               #
;#     Alpha          = is the field an alphanumeric field                     #
;#     AlwaysSign     = is the field always signed                             #
;#     Digits         = number of significant digits                           #
;#     International  = is international notation used                         #
;#     PointNoPennies = a field uses a decimal point but no decimal places     #
;#     Wrapped        = is the alpha field word wrapped.                       #
;#   Use the Reset selection from the CopyFld menu to release these variables. #
;#                                                                             #
;#   This utility is offered to Paradox users free of charge.                  #
;#   You may give copies of CopyFld to any other Paradox users who have a      #
;#   use for it as long as these copies have not been modified in any way.     #
;#                                                                             #
;#                      < < < <   DISCLAIMER   > > > > >                       #
;#   Data Works of Houston does not guarantee the suitability of CopyFld for   #
;#   any purpose and cannot be held liable for any damage caused either        #
;#   directly or indirectly through the use of CopyFld.                        #
;#                                                                             #
;###############################################################################
PRIVATE Errorproc, Choice, StartingColumn, i, CurrentColumn, Position, MenuMove,
        StartColumn, TempDigits, Insert, NameLength, ChoiceKey, a, VerInfo
errorproc = "CopyFldError"

IF MEMLEFT() < 30000
   THEN
      QUIT "Not enough memory available to use CopyFld."
ENDIF

STYLE ATTRIBUTE 79
@ 24,61 ?? " Loading CopyFld"
STYLE ATTRIBUTE 207
?? "..."
STYLE

PROC CopyFldError()
   MENU ESC
   QUIT "Sorry, CopyFld cannot work in this situation."
ENDPROC

PROC GetFieldSize()
Private DecimalPoint, WholeNumberSeparator,TakeOff,LeftOn, Temp
Digits = 0
Decimals = 0

SWITCH
   CASE MATCH(UPPER(FieldFormat),"..m.."):
      ;This is a date or time field, do nothing.
      Return
   CASE MATCH(FieldFormat,"..a.."):
      ;This is an alphanumeric field.  Set Digits to length
      Digits = LEN(FieldFormat)
      RETURN
   OTHERWISE:
      MENU {Field} {Reformat} ENTER
      IF MenuChoice() = ""
         THEN
            ;Report fields Page or #Record is selected, set Digits to length.
            MENU ESC
            Digits = LEN(FieldFormat)
            RETURN
         ELSE
            ;See if the numeric or currency field is in InternationalConvention
           {International}
           International = MenuChoice() = "InternationalConvention"
           MENU ESC
      ENDIF
ENDSWITCH
;Continue here only with numeric or currency fields
IF International
   THEN
      DecimalPoint = ","
      WholeNumberSeparator = "."
   ELSE
      DecimalPoint = "."
      WholeNumberSeparator = ","
ENDIF

Commas = NOT (SEARCH(WholeNumberSeparator, FieldFormat) = 0)
LeftOn = FieldFormat
TakeOff = " "

WHILE LEN(LeftOn) > 0
   Temp = MATCH(LeftOn,"@..", Takeoff, Lefton)
   SWITCH
      CASE TakeOff = "9":
         Digits = Digits + 1
      CASE TakeOff = DecimalPoint:
         PointNoPennies = NOT MATCH(LeftOn,"..9..")
         QUITLOOP
      OTHERWISE:
   ENDSWITCH
ENDWHILE

WHILE LEN(LeftOn)>0
   Temp = MATCH(LeftOn,"@..",Takeoff, LeftOn)
   SWITCH
      CASE TakeOff = "9":
         Decimals = Decimals + 1
      OTHERWISE:
   ENDSWITCH
ENDWHILE
ENDPROC ;GetFieldSize()

Proc GroupedStatus()
    IF LEN(NameOfField) > 10
       THEN
          IF SUBSTR(NameOfField,LEN(NameOfField)-10,
             Len(NameOfField)) =", per group"
             THEN
                NameOfField=SUBSTR(NameOfField, 1, LEN(NameOfField)-11)
                RETURN True
             ELSE
                RETURN False
          ENDIF
       ELSE
          RETURN False
    ENDIF
ENDPROC ;GroupedStatus()

PROC SummaryType()    ;check for a [ or a "
   IF SEARCH("[", NameOfField)=0  AND SEARCH("\"", NameOfField)=0
      THEN
         RETURN "Regular"
      ELSE
         RETURN "Calculated"
   ENDIF
ENDPROC ;SummaryType()

PROC WaitKey()
Private Temp
   WHILE CHARWAITING()
      Temp = GETCHAR()
   ENDWHILE
   Temp = GETCHAR()
ENDPROC

Proc IsRegular()
Private CurrentItem
;This procedure verifies that the field was a regular field,
;not a calculation containing a variable or constant.
CurrentItem = MenuChoice()
WHILE UPPER(NameOfField) <> UPPER(MenuChoice())
   KEYPRESS MenuMove
   IF MenuChoice() = CurrentItem   ;Opps, we got a variable
      THEN                         ;or a constant
         PlaceType = "Calculated"
         RETURN False
   ENDIF
ENDWHILE
ENTER
RETURN True
ENDPROC  ;IsRegular()

PROC ApplyJustification(AdjustCursorPosition)
  IF NOT Justify = "None"
     THEN
        IF AdjustCursorPosition
           THEN
              LEFT
        ENDIF
        MENU {Field} {Justify} ENTER SELECT Justify
  ENDIF
ENDPROC

PROC GetDateFormat()
  WHILE NOT SUBSTR(MenuChoice(),
      SEARCH(" ",MENUCHOICE())+1 ,
      LEN(MENUCHOICE())
      - SEARCH(" ",MENUCHOICE())+1 ) = FieldFormat
     KEYPRESS MenuMove
  ENDWHILE
  ENTER
  ENTER
ENDPROC

;# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
;#                                                                         #
;#                       M A I N   P R O G R A M                           #
;#                                                                         #
;# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

@ 24,0 CLEAR EOL
SWITCH
   CASE (NOT SYSMODE()= "Report") OR
      (NOT IsAssigned(NameOfField) AND FIELDINFO()=""):
      SHOWMENU
         "Help"   :"Display help on how to use CopyFld.",
         "Info"   :"Display credits and CopyFld information.",
         "Reset"  :"Reset all variables used by CopyFld and exit.",
         "Display":"Display the contents of variables used by CopyFld.",
         "Exit"   :"Return to what you where doing."
         TO Choice

   CASE IsAssigned(NameOfField) AND FIELDINFO()="":
      SHOWMENU
         "Paste"  :"Paste the previously copied field "+
                   "to the location at the cursor.",
         "Help"   :"Display help on how to use CopyFld.",
         "Info"   :"Display credits and CopyFld information.",
         "Reset"  :"Reset all varaibles used by CopyFld and exit.",
         "Display":"Display the contents of variables used by CopyFld.",
         "Exit"   :"Return to the report generator without "+
                   "copying or pasting a field."
         TO Choice
   OTHERWISE:
      SHOWMENU
         "Copy"   :"Copy the field at the cursor.",
         "Help"   :"Help on how to use CopyFld version 1.3.",
         "Info"   :"Credits and additional CopyFld version 1.3 information.",
         "Reset"  :"Reset all variables used by CopyFld and exit.",
         "Display":"Display the contents of variables used by CopyFld.",
         "Exit"   :"Return to the report generator without copying the field."
         TO Choice
ENDSWITCH

SWITCH

   CASE Choice="Exit" OR Choice="Esc":
      CLEAR
      QUIT  "Thank you for using CopyFld"

   CASE Choice = "Display":
      CANVAS OFF
      PAINTCANVAS ATTRIBUTE 7  3,16,23,69
      PAINTCANVAS FILL(" ") ATTRIBUTE 78 2,14,22,67
      CANVAS ON
      STYLE ATTRIBUTE 78
      @ 2,17 ?? "Variable assignments from the last field copied"
      SETMARGIN 15
      @ 3,0
      ? "Alpha          = "
      ? "AlwaysSign     = "
      ? "Commas         = "
      ? "Currency       = "
      ? "Decimals       = "
      ? "Digits         = "
      ? "FieldFormat    = "
      ? "Grouped        = "
      ? "International  = "
      ? "Justify        = "
      ? "NameOfField    = "
      ? "NegParens      = "
      ? "PlaceType      = "
      ? "PointNoPennies = "
      ? "StatType       = "
      ? "SummaryType    = "
      ? "Wrapped        = "
      ? "WrappedTimes   = "
      STYLE ATTRIBUTE 79
      SETMARGIN 32
      @ 3,0
      IF ISASSIGNED(Alpha) THEN ? FORMAT("W33,AL",STRVAL(Alpha))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(AlwaysSign) THEN ? FORMAT("W33,AL",STRVAL(AlwaysSign))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(Commas) THEN ? FORMAT("W33,AL",STRVAL(Commas))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(Currency) THEN ? FORMAT("W33,AL",STRVAL(Currency))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(Decimals) THEN ? FORMAT("W33,AL",STRVAL(Decimals))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(Digits) THEN ? FORMAT("W33,AL",STRVAL(Digits))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(FieldFormat) THEN ? FORMAT("W32,AL",STRVAL(FieldFormat))
         IF LEN(FieldFormat) > 32 THEN ?? "..." ENDIF
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(Grouped) THEN ? FORMAT("W33,AL",STRVAL(Grouped))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(International) THEN ? FORMAT("W33,AL",STRVAL(International))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(Justify) THEN ? FORMAT("W33,AL",STRVAL(Justify))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(NameOfField) THEN ? FORMAT("W32,AL",STRVAL(NameOfField))
         IF LEN(NameOfField) > 32 THEN ?? "..." ENDIF
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(NegParens) THEN ? FORMAT("W33,AL",STRVAL(NegParens))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(PlaceType) THEN ? FORMAT("W33,AL",STRVAL(PlaceType))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(PointNoPennies) THEN ? FORMAT("W33,AL",STRVAL(PointNoPennies))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(StatType) THEN ? FORMAT("W33,AL",STRVAL(StatType))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(SummaryType) THEN ? FORMAT("W33,AL",STRVAL(SummaryType))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(Wrapped) THEN ? FORMAT("W33,AL",STRVAL(Wrapped))
         ELSE ? "Unassigned" ENDIF
      IF ISASSIGNED(WrappedTimes) THEN ? FORMAT("W33,AL",STRVAL(WrappedTimes))
         ELSE ? "Unassigned" ENDIF
      STYLE ATTRIBUTE 78
      SETMARGIN 0
      @22,28 ?? "Press any key to continue"
      STYLE
      a=GETCHAR()
      QUIT

   CASE Choice = "Help":
      CLEAR
      @ 0,0
      STYLE REVERSE
      ?? FORMAT("W80,AC","Using CopyFld (version 1.3)")
      STYLE
      @ 1,0
      TEXT

      To copy a field in the report generator:
           Move your cursor onto the field you want to copy.
           Play CopyFld and select {Copy} from the CopyFld menu.

      To paste a field that you have copied:
           Move your cursor to the position in the report where you
          want place the copied field.
           Play CopyFld and select {Paste} from the CopyFld menu.

      Note: The following special conditions affect how CopyFld works:
           CopyFld assumes that any field named "Current date",
            "Current time", "Current page number", or
            "Current record number" is a report field, not a regular field.
           Any field whose name begins "Maximum for", "Minimum for",
            "Average for", "Average of", "Total for", or "Count of" will
            be assumed to be a summary field, not a regular field.

      The following variables are defined by CopyFld.  Select Reset to release.
         Decimals, WrappedTimes, Justify, PlaceType, NegParens, Commas,
         NameOfField, Currency, FieldFormat, Alpha, AlwaysSign, Digits,
         International, Wrapped, Grouped, SummaryType, StatType, PointNoPennies
      ENDTEXT
      STYLE REVERSE
      ? FORMAT("W80,AC","Press any key to continue")
      STYLE
      WaitKey()
      QUIT  "Thank you for using CopyFld"

   CASE Choice = "Info":
      CLEAR
      @ 0,0
      STYLE REVERSE
      ?? FORMAT("W80,AC","Credits and Information for CopyFld (version 1.3)")
      STYLE
      @ 1,0
      TEXT
         Written by Cary Jensen of Data Works of Houston, co-author of
             PAL for Paradox 4 Made Easy (Osborne/McGraw-Hill) and
                 The Paradox Instant Reference books (Sybex).

   Data Works of Houston provides Paradox and Paradox for Windows training,
   consultation, and application development.  Customized classes available.
                          ķ
                            Data Works of Houston  
                               (713) 359-3311      
                          Ľ
                                  *  *  *  *
      Invoke CopyFld using a "hot-key."  This is done using the SETKEY command.
      From your init script, or from the {MiniScript} selection from the
      PAL menu [ALT][F10], play the following command at the beginning of
      your Paradox session:

      SETKEY "F23" READLIB "C:\\PDOX35\\CopyTool" CopyFld CopyFld()

      After you issue this command, CopyFld can be invoked by pressing CTRL-F3.
      To assign CopyFld to any other key, replace "F23" in the SETKEY command
      with any other valid PAL keycode or key name.  If Copytool.lib is stored
      in a directory other than C:\PDOX35, use that path in the SETKEY command.
      ENDTEXT
      STYLE REVERSE
      ? FORMAT("W80,AC","Press any key to continue")
      STYLE
      WaitKey()
      QUIT  "Thank you for using CopyFld"

   CASE Choice="Reset":
      RELEASE VARS Decimals, WrappedTimes, Justify, PlaceType, NegParens,
      Commas, NameOfField, Currency, FieldFormat, Alpha, AlwaysSign,
      Digits, International, Wrapped, Choice, StartingColumn, i, CurrentColumn,
      StartColumn, NameLength, Grouped, SummaryType, StatType, PointNoPennies

      QUIT "All CopyFld variables have been reset."

   CASE Choice="Copy":
      IF FIELDINFO()=""
         THEN
            QUIT "Your cursor must be on a field."
         ELSE
            STYLE ATTRIBUTE 79
            @ 24,63 ?? " Copying field"
            STYLE ATTRIBUTE 207
            ?? "..."
            STYLE
            Alpha = False
            Commas = False
            Currency = False
            International = False
            PointNoPennies = False
            Justify = "None"
            NegParens = False
            Wrapped = False
            WrappedTimes = 0
            NameOfField=FIELDINFO()
            RELEASE VARS Grouped, StatType, SummaryType
      ENDIF

      ;move to the first character of the field
      StartColumn=COLNO()
      LEFT
      ;If FIELDINFO is no longer the name, we moved off the field.
      ;If the column hasn't changed, we cannot move LEFT any more.
      WHILE FIELDINFO()=NameOfField AND NOT StartColumn=COLNO()
         StartColumn=COLNO()
         LEFT SYNCCURSOR
      ENDWHILE
      ;If cursor moved with last LEFT, move
      ;back onto the field with a RIGHT
      IF NOT StartColumn=COLNO()
         THEN
            RIGHT
      ENDIF
      NegParens = CURSORCHAR()="("
      AlwaysSign = CURSORCHAR() = CHR(241)
      FieldFormat=""
      ;Determine the field format.
      WHILE FIELDINFO()=NameOfField
         FieldFormat=FieldFormat+CURSORCHAR()
         StartingColumn=COLNO()
         RIGHT
         IF StartingColumn=COLNO()
            THEN
               QUITLOOP
         ENDIF
      ENDWHILE
      ;Re-wind to the first of the field
      FOR i FROM 1 to LEN(FieldFormat)
         LEFT
      ENDFOR

      Alpha = CURSORCHAR() = "A"
      ;If the field is alphanumeric
      ;see if the field is wordwrapped
      IF Alpha
         THEN
            IF NOT SEARCH(", wrap:",NameOfField) = 0
               THEN
                  Wrapped = True
                  Position = SEARCH(", wrap:",NameOfField)
                  WrappedTimes = NumVal(SubStr(NameOfField,
                                 Position+7, LEN(NameOfField)))
                  IF WrappedTimes = "Error"
                     THEN WrappedTimes = "v"
                  ENDIF
                  NameOfField = SubStr(NameOfField, 1, Position-1)
            ENDIF
      ENDIF

      ;Determine if the field is justified.
      NameLength=LEN(NameOfField)
      SWITCH

         CASE NameLength > 5 AND SUBSTR(NameOfField,
            NameLength-5, NameLength) = ", left":
            NameOfField= SUBSTR(NameOfField, 1, NameLength-6)
            Justify = "Left"
         CASE NameLength > 6 AND SUBSTR(NameOfField,
            NameLength-6, NameLength) = ", right":
            NameOfField= SUBSTR(NameOfField, 1, NameLength-7)
            Justify = "Right"

         CASE NameLength > 7 AND SUBSTR(NameOfField,
            NameLength-7, NameLength) = ", center":
            NameOfField= SUBSTR(NameOfField, 1, NameLength-8)
            Justify = "Center"

         OTHERWISE:
            Justify = "None"

      ENDSWITCH


      ;Check for the various summary field types.  If the
      ;field is a summary field, assign the appropriate variables
      SWITCH

         CASE SUBSTR(NameOfField,1,12) = "Maximum for ":
            StatType="High"
            PlaceType="Summary"
            Grouped=GroupedStatus()
            SummaryType=SummaryType()
            NameOfField = SubStr(NameOfField,13, LEN(NameOfField))
            GetFieldSize()

         CASE SUBSTR(NameOfField,1,12) = "Minimum for ":
            StatType="Low"
            PlaceType="Summary"
            Grouped=GroupedStatus()
            SummaryType=SummaryType()
            NameOfField = SubStr(NameOfField,13, LEN(NameOfField))
            GetFieldSize()

         CASE SUBSTR(NameOfField,1,12) = "Average for ":
            StatType="Average"
            PlaceType="Summary"
            Grouped=GroupedStatus()
            SummaryType=SummaryType()
            NameOfField = SubStr(NameOfField,13, LEN(NameOfField))
            GetFieldSize()

         CASE SUBSTR(NameOfField,1,11) = "Average of ":
            StatType="Average"
            PlaceType="Summary"
            Grouped=GroupedStatus()
            SummaryType=SummaryType()
            NameOfField = SubStr(NameOfField,12, LEN(NameOfField))
            GetFieldSize()

         CASE SUBSTR(NameOfField,1,10) = "Total for ":
            StatType="Sum"
            PlaceType="Summary"
            Grouped=GroupedStatus()
            SummaryType=SummaryType()
            NameOfField = SubStr(NameOfField,11, LEN(NameOfField))
            GetFieldSize()

         CASE SUBSTR(NameOfField,1,9) = "Count of ":
            StatType="Count"
            PlaceType="Summary"
            Grouped=GroupedStatus()
            SummaryType=SummaryType()
            NameOfField = SubStr(NameOfField,10, LEN(NameOfField))
            GetFieldSize()

         ;The field is not a summary field.  It must then either
         ;be a regular field or a calculated field.
         OTHERWISE:
            IF SummaryType()="Calculated"
               THEN
                  PlaceType="Calculated"
                  GetFieldSize()
               ELSE
                  ;If the field is a regular field, make sure
                  ;that it is not a report field.
                  SWITCH

                     CASE NameOfField = "Current date":
                        PlaceType="Report field"
                        StatType="Date"

                     CASE NameOfField = "Current time":
                        PlaceType="Report field"
                        StatType="Time"

                     CASE NameOfField = "Current page number":
                        GetFieldSize()
                        PlaceType="Report field"
                        StatType="Page"

                     CASE NameOfField = "Current record number":
                        GetFieldSize()
                        PlaceType="Report field"
                        StatType="#Record"
                        Grouped=False

                     CASE NameOfField = "Current record number, per group":
                        GetFieldSize()
                        PlaceType="Report field"
                        Grouped=True
                        StatType="#Record"

                     OTHERWISE: ;its a regular field
                        PlaceType="Regular"
                        GetFieldSize()
                  ENDSWITCH
            ENDIF
      ENDSWITCH

      QUIT "Field has been copied"

   CASE Choice="Paste":
      IF NOT CURSORCHAR()=" "
         THEN
            QUIT "Cursor must be on a blank space in order to paste a field."
      ENDIF
      STYLE ATTRIBUTE 79
      @ 24,63 ?? " Pasting field"
      STYLE ATTRIBUTE 207
      ?? "..."
      STYLE
      Insert = IsInsertMode()
      ;Turn off insert mode if it is on.
      IF Insert
         THEN
            INS
      ENDIF

      ;Check for mode compatibility
      IF VERSION() > 3.5
         THEN
            MENU HOME
            IF MENUCHOICE()= "Field"
               THEN
                  MenuMove = "RIGHT"
               ELSE
                  MenuMove = "DOWN"
            ENDIF
         ELSE
            MenuMove = "RIGHT"
      ENDIF
      ESC

      WHILE True
         SWITCH
            CASE PlaceType="Regular":
               MENU {Field} {Place} {Regular}
                  ;It is possible that a field could be identified
                  ;as a regular field although it is really a calculated
                  ;field that uses functions or variables.  IsRegular()
                  ;ensures that the field is a regular field.  If it is not,
                  ;PlaceType is assigned "Calculated" and False is returned
                  IF NOT IsRegular()
                     THEN
                        LOOP
                  ENDIF

                  IF MENUCHOICE()="1) mm/dd/yy"
                     ;The field is a date field.  Select the correct format.
                     THEN
                        GetDateFormat()
                        ApplyJustification(True)
                        ;All done for regular field of type date.
                        QUITLOOP
                  ENDIF
                  ;If the field is not of the type Date, processing
                  ;will continue after the ENDSWITCH.

            CASE PlaceType="Report field":
               MENU {Field} {Place} SELECT StatType
               SWITCH
                  CASE StatType="Date":
                     ;If the report field is Date,  select the correct format.

                     GetDateFormat()
                     ApplyJustification(True)
                     QUITLOOP

                  CASE StatType="Time":
                     ;If the report field is Time,  select the correct format.
                     WHILE NOT MATCH(MENUCHOICE(), ".."+ FieldFormat +"..")
                        KEYPRESS MenuMove
                     ENDWHILE
                     ENTER
                     ENTER
                     ApplyJustification(True)
                     QUITLOOP

                  CASE StatType="#Record":
                        IF Grouped
                           THEN
                              {Per-Group}
                           ELSE
                              {Overall}
                        ENDIF
                  OTHERWISE:
               ENDSWITCH
               ;If the report field is type #Record, placement will continue
               ;after the ENDSWITCH

            CASE PlaceType="Summary":
               MENU {Field} {Place} {Summary}
               SELECT SummaryType
               IF SummaryType="Calculated"
                  THEN
                     TYPEIN NameOfField ENTER
                  ELSE
                     SELECT NameOfField
               ENDIF
               SELECT StatType
               IF Grouped
                  THEN
                     {PerGroup}
                  ELSE
                     {Overall}
               ENDIF
               ;If the summary field is type date, select correct format.
               IF MENUCHOICE()="1) mm/dd/yy"
                  THEN
                     GetDateFormat()
                     ApplyJustification(True)
                     QUITLOOP
               ENDIF

            CASE PlaceType="Calculated":
               MENU {Field} {Place} {Calculated}
               TYPEIN NameOfField ENTER
               ;If calculated field is date, select correct format.
               IF MENUCHOICE()="1) mm/dd/yy"
                  THEN
                     GetDateFormat()
                     ApplyJustification(True)
                     QUITLOOP
               ENDIF

         ENDSWITCH


         StartingColumn = COLNO()
         ENTER
         CurrentColumn = COLNO()
         HOME
          IF NOT StartingColumn=COLNO() ;AND NOT AlwaysSign
             THEN
                Currency = True
             ELSE
                Currency = False
          ENDIF

         ;=================================;
         ;We know that the field is a number,
         ;currency, or alphanumeric field.
         ;=================================;

         For I from 2 to Digits
            Right
         ENDFOR
         ENTER

         If Not Decimals=0
            THEN
               HOME
               RIGHT
               FOR I FROM 1 TO Decimals
                   RIGHT
               ENDFOR
            ELSE
                IF Currency
                   THEN
                      HOME
                ENDIF
               IF PointNoPennies
                 THEN
                    RIGHT
               ENDIF
         ENDIF

         IF NOT (Alpha OR PlaceType = "Report field")
            THEN
               ;The field is a currency or numeric field.
               ;We need to issue one last ENTER.
               ENTER
               ;Now, we need to set the appropriate formatting features.
               ;=================================;
               ;Setting digit/decimal delimiter notation.
               MENU {Field} {Reformat} ENTER {International}
               IF International
                  THEN
                     {InternationalConvention}
                  ELSE
                     {U.S.Convention}
               ENDIF
               ;Setting commas.
               MENU {Field} {Reformat} ENTER {Commas}
               IF Commas
                  THEN
                     {Commas}
                  ELSE
                     {NoCommas}
               ENDIF
               ;Setting sign convention.
               MENU {Field} {Reformat} ENTER {Sign-Convention}
               IF NOT NegParens
                  THEN
                     IF AlwaysSign
                        THEN
                           {AlwaysSign}
                        ELSE
                           {NegativeOnly}
                     ENDIF
                 ELSE
                   {ParenNegative}
               ENDIF
         ENDIF

         ApplyJustification(False)

         IF Wrapped
            THEN
               MENU {Field} {WordWrap} ENTER CTRLBACKSPACE
               TYPEIN WrappedTimes ENTER
         ENDIF

         ;All done!
         QUITLOOP
      ENDWHILE

      ;If insert mode was on, turn it back on.
      IF Insert
         THEN
            INS
      ENDIF

      RELEASE VARS Choice, StartingColumn, i, ii, CurrentColumn,
                   StartColumn, TempDigits, Insert

      QUIT "Field has been pasted"
ENDSWITCH

ENDPROC ;------------------------- CopyFld ------------------------------
IF NOT ISFILE(DIRECTORY()+"CopyTool.lib")
   THEN
      CREATELIB DIRECTORY()+"CopyTool"
ENDIF
Writelib DIRECTORY()+"copytool" CopyFld
QUIT "COPYTOOL.LIB is stored in "+DIRECTORY()+"  Read CopyFld.sc"
+" for directions on how to play CopyFld.  Press any key to continue."

